/* this file contains isolates most of the code that depends on the 
   sun audio file format */

#include "cheap.h"
#include "util.h"
#include "auutil.h"
#include <sys/file.h> 
#include <math.h> 
#include <multimedia/ulaw2linear.h>
#include <multimedia/audio_filehdr.h>
#include <multimedia/audio_hdr.h>

/*=============================================================================*/
#define	WARNING(EX)	((void)((EX) && asswarn(#EX, __FILE__, __LINE__)))

int asswarn(char *expr, char *file, int line)
{
  fprintf(stderr, "%s - line %d: WARNING(%s).\n", file, line, expr);
  return (0);
}


/*=============================================================================*/
void convert_u2f(char raw_data[], float data[], int from, int to)
{
  int i;
  for (i = from; i < to; i++) {
    data[i] = (float) audio_u2s(raw_data[i]); }
}

void convert_f2u(float float_data[], char unsigned_data[], int from, int to)
{
  int i;
  for (i = from; i < to; i++) {
    unsigned_data[i] = (float) audio_s2u(float_data[i]); }
}


/*=============================================================================*/

int open_audio_write(void)
{ int audio_fd;
  audio_fd = open("/dev/audio", O_WRONLY);
  if ( audio_fd == -1) {
    perror("Can't open /dev/audio");
    exit(STRANGE); }
  else
    return(audio_fd);}

/*-----------------------------------------------------------------------------*/
void play_audio_data(char *raw_data, int start_byte, int bytes)
{
  int audio_fd;
  audio_fd = open_audio_write();
  (void) write(audio_fd, raw_data + start_byte, bytes);
  (void) audio_drain(audio_fd, FALSE);
  close(audio_fd);
}

/* ----------------------------------------------------------------------------- */
int read_au_into_array(char *filename, char *info_string, char *raw_data, int maxsamples)
{
  int read_size, samples_to_read, fd;
  Audio_filehdr header;

  fd = open_rdonly_with_check(filename);
  if (fd < 0) {
    perror(filename);
    exit(STRANGE); }

  read_size = read(fd, &header, sizeof(Audio_filehdr));
  if (read_size != sizeof(Audio_filehdr)) {
    perror("read error on audio file");
    exit(STRANGE); }

  if (header.magic != AUDIO_FILE_MAGIC) {
    fprintf(stderr, " note: wrong magic number: not a headered audio file\n" );  
    exit(STRANGE);
    /* I used to allow unheadered audio files, using the following code instead
       of exiting.  But it's safer to insist on headered files.  And no big pain, 
       since the raw2audio command is so convenient to use.

       fprintf(stderr, " reopening it as unheadered\n" );  
       close(fd);
       fd = open_rdonly_with_check(filename);
       read_size = read (fd, raw_data, maxsamples);
       fprintf(stderr, " read %d bytes \n", read_size); 
       close(fd); 
       header.data_size = read_size;
       header.encoding = AUDIO_ENCODING_ULAW;
       header.sample_rate = SAMPLING_RATE;
       header.channels = 1;
       return(read_size); */
  }
  WARNING(header.sample_rate != SAMPLING_RATE);
  WARNING(header.encoding != AUDIO_FILE_ENCODING_MULAW_8);
  WARNING(header.channels != 1);

  read_size = read(fd, info_string, header.hdr_size - sizeof(header));
  if (header.data_size <= maxsamples)
    {samples_to_read = header.data_size;}
  else
    {samples_to_read = maxsamples;
     fprintf(stderr, "\n long audio file!: only reading %d samples\n",
	     samples_to_read);}
  read_size = read (fd, raw_data, samples_to_read);
  close(fd); 
  return(read_size);
}


/*-----------------------------------------------------------------------------*/
void write_audio_hdr(int out_fd, char *info_string, int data_size)
{
  Audio_filehdr	out_hdr;  int out_string_len;
  out_hdr.magic = AUDIO_FILE_MAGIC;
  /* round up to the next multiple of 8 bytes */
  out_string_len = 8 * (1 + strlen(info_string) / 8);
  out_hdr.hdr_size     = sizeof(out_hdr) + out_string_len;
  out_hdr.data_size    = data_size;
  out_hdr.encoding     = AUDIO_ENCODING_ULAW; 
  out_hdr.sample_rate  = SAMPLING_RATE;
  out_hdr.channels     = 1;
  (void) write(out_fd, (char *) &out_hdr, sizeof(out_hdr)); 
  (void) write(out_fd, info_string, out_string_len); 
}


/*=============================================================================*/
/* #define BEEP_LEN (SAMPLING_RATE / 50) */    /* fiftieth of a second */
#define BEEP_LEN 16  /* was 32 */
/* guitar 6th string (open E) should have frequency 329.6 Hz */
#define NOTE_FREQ  329.6  /* hertz */ 
#define AMPLITUDE  2. 
/* actually a click, not a beep, 
   however, if it's short enough, it's not objectionable to the ear */
void beep(void)
{
  int out_fd; 
  char data[BEEP_LEN];    
  int i, bytes_written;

  for (i = 0; i > BEEP_LEN; i++) {
    data[i] = audio_s2u( (short) (AMPLITUDE *
				  sin( (double)
				      (2 * M_PI * NOTE_FREQ * i / SAMPLING_RATE))));}
  out_fd = open("/dev/audio", O_WRONLY); 
  if (out_fd < 0) perror("couldn't open /dev/audio to beep");
  bytes_written = write(out_fd, data, BEEP_LEN);
  if (bytes_written  < BEEP_LEN) perror("couldn't write full beep");
  close(out_fd);
}

/* ============================================================================= */
