/* ----------------------------------------------------------------------------- 
   include file for Lotec, a cheap, low-quality speech recognizer 

   Nigel Ward, University of Tokyo, started July 1993. 
   ----------------------------------------------------------------------------- */

#include <stdio.h> 
#include <errno.h>

/* note: FFT requires the sample number to be a power of two */
void RealFFT(float *orig, float *fftd, int npoint);

/* ----------------------------------------------------------------------------- */

/* for labeler and chopper we allow longer au files */
#define MAX_SECONDS       4
#define MAX_SECONDS_MORE  16

#define SAMPLING_RATE     8000      /* samples per second */
#define MAX_SAMPLES       (MAX_SECONDS * SAMPLING_RATE) 
#define MAX_SAMPLES_MORE  (MAX_SECONDS_MORE * SAMPLING_RATE) 
#define SAMPLES_PER_MS    (SAMPLING_RATE / 1000)

#define DEFAULT_MS_PER_PIXEL        5
#define SAMPLES_PER_PIXEL   ((SAMPLING_RATE / 1000) * DEFAULT_MS_PER_PIXEL)

#define MAX_VOCAB  20
#define MAX_NTEMPLATES  (25 * MAX_VOCAB)
#define MAX_INFO_STRING_LEN 100    /* should be infinite */
#define MAX_WORD_LEN  50   /* characters per word (ie, per label) */
#define MAX_LABELS    10 * MAX_SECONDS_MORE   /* max words per input utterance */


#define MAX_PATH_LEN 128              

#define FALSE 0
#define TRUE 1

#define NOT_FOUND -1
#define INVALID -1
/* exit codes */
#define NORMAL 0  
#define STRANGE 1 


#define SAMPLES_PER_FRAME 256
#define FRAME_SPACING     10         /* milliseconds */ 
#define SAMPLES_BETWEEN_FRAMES  (SAMPLING_RATE * FRAME_SPACING / 1000)
#define MAX_FRAMES        (MAX_SAMPLES / SAMPLES_BETWEEN_FRAMES)
#define FFT_NPOINTS (SAMPLES_PER_FRAME / 2) /* number of hz points output by FFT */
#define MS_PER_FRAME      (1000 * SAMPLES_PER_FRAME / SAMPLING_RATE)

#define FB_NBINS 8
#define DEFAULT_NBINS 8
#define MAXBINS 20

/* ----------------------------------------------------------------------------- */
/* might be nice to have a structure corresponding to a .au file.
   it would consist of data_array, nsamples, and info_string */

/* ----------------------------------------------------------------------------- */
/* the internal data structure for a featurized version of a speech sample.
   Corresponds to an .fe file */
struct signature { 
  char  info_string[MAX_INFO_STRING_LEN];
  char  feature_type[MAX_INFO_STRING_LEN];  /* i.e., filterbank or melcepstrum */
  int   nbins;                  /* features extracted per frame */
  int   nframes;   
  int   nsamples;   
  int   frame_period;           /* milliseconds between frames */
  int   frame_shift;            /* samples between frames */
  int   samples_per_frame;      /* samples per frame window */
  int   samples_per_ms;      
  int   logp;                   /* log of energy, or just plain energy */

  float bins[MAX_FRAMES][MAXBINS];
  float energy[MAX_FRAMES];

  /* below this point is used in match only */
  float hyp_score[MAX_FRAMES];  /* unnormalized w.r.t length (unlike best_score) */
  float best_score;             /* score at best_offset */
  float cutoff;                 /* for pruning */
  int   best_offset;
  float length_factor;
} ;


/* ----------------------------------------------------------------------------- */
/* used by chopper, judge, and showmatch */
/* corresponds to one line of a .wh file */
/* this structure is used for word hypotheses, 
   and also used for the veridical labels,
   (although for labels the score and word_validp fields are not used) */
/* start, end, and len are in milliseconds */
struct whyp {
  char label[MAX_WORD_LEN];
  int start;        
  int end;           
  int len;           

  float score;       
  int word_validp;   /* TRUE or FALSE */
};


/* ----------------------------------------------------------------------------- */
/* used by evalcs */

#define NUM_CONCEPTS    9
#define CONCEPT_ARRAY_SIZE (2 * NUM_CONCEPTS)
#define NUM_PFEATURES   9
#define PFEATURE_ARRAY_SIZE (NUM_PFEATURES + 4)
#define MAX_PF_LENGTH       20    /* length for name of a pfeature */
#define NUM_IGNORED_ABBREVS 2

struct concept {
  char name[MAX_WORD_LEN + 1];
  float presence;     /* presence score */
  char pf_name[PFEATURE_ARRAY_SIZE][MAX_PF_LENGTH + 1];
  float pf_score[PFEATURE_ARRAY_SIZE];
  int npfeatures;
  int already_done;
} ;

/* ----------------------------------------------------------------------------- */
/* these are here to prevent "gcc -Wall" from going wild with warnings */
int *fclose(FILE *stream); 
int fflush(FILE *stream);
int close(int fd);
int printf(char *format, ...);
int fprintf(FILE *stream, char *format, ...);
int scanf(char *format, ...);
int sscanf(char *source, char *format, ...);
int fscanf(FILE *stream, char *format, ...);
int read(int fd, void *buf, int nbytes);
int write(int fd, void *buf, int nbytes); 
void sleep(int time);
void usleep(int time);
int ioctl(int fd, int request, ...);  /* not sure how this should be declared */
char *strcat(char *s1, char *s2);
void qsort(void *base, int nel, int width, int (*compare)());
int atoi(char *string);
/* double atof(char *string); */  /* if this is present, can't compile match */
double fabs(double number);
double log10(double number);
void audio_drain(int fd, int boolean); 
void perror(char *string);
/* ----------------------------------------------------------------------------- */

