#include "cheap.h"
#include "util.h"
#include "fmeat.h"

/* this file converts .ht files to .fe files.
   .ht (HTK format) files are created by
      HCode -F SUNAU8 -e -h -m -n 8 -k 0.97 -w 32.0 testfile.au testfile.ht
      (ie, mel-cepstrum files, 10 ms frame-spacing.)
   .fe files can be read into the matcher */

/* FILE *open_out_file(char*, char*); */
struct signature sig;

void main(int argc, char *argv[]);
void convert_file(int in_fd, FILE *out_fp, char *pathstring);

/*-----------------------------------------------------------------------------*/
void main(int argc, char *argv[])
{
  int i;
  int in_fd; FILE *out_fp;

  if (argc == 1) {
    /* do it from stdin to stdout */
    convert_file(0, stdout, "stdin.au");
  }
  else {
    for(i = 1; i < argc; i++) {
      in_fd = open_rdonly_with_check(argv[i]);
      if (in_fd < 0) { exit(STRANGE); }
      else {
	/* printf( "\n processing path = %s \n", argv[i]); */
	out_fp = fopen_out_file(argv[i],".fe");
	convert_file(in_fd, out_fp, argv[i]); 
	close(in_fd);
	fclose(out_fp); } } }
}

/*-----------------------------------------------------------------------------*/
void convert_file(int in_fd, FILE *out_fp, char *pathstring)
{
  int read_size, i, j;
  long int nsamples, samp_period; 
  short int samp_size, samp_kind;
  int nfeatures;
  char idstring[100];
  float raw_data[MAX_SAMPLES * 17];  /* assume no more than 17 features per frame */

  /* read in header */
  read_size = read(in_fd, &nsamples, 4);
  read_size = read(in_fd, &samp_period, 4);
  read_size = read(in_fd, &samp_size, 2);
  read_size = read(in_fd, &samp_kind, 2);
  
  nfeatures = (samp_size / 4) - 1;   /* -1 since energy is not counted as a feature */
  /* printf(" nsamples %d, samp period %5.1f (ms), samp_size %d (bytes), sample type code %d \n",  
	 nsamples, samp_period / 10000.0, samp_size, samp_kind); */
  if(samp_kind != 70) {  /* 64 + 6 means MFCC + energy */
    fprintf(stderr, "unknown feature format. %d \n", samp_kind);
    fprintf(stderr, "Was the file created with the wrong parameters to HCode?"); 
    /* exit or something */
  }

  /* write out header */
  basefilename(idstring, pathstring);

  printf(" pathstring is `%s', idstring is `%s'\n", pathstring, idstring);
  
  sig.nbins = nfeatures;
  sig.frame_period = (int) samp_period / 10000.0;
  strcpy(sig.info_string, idstring);
  strcpy(sig.feature_type, "htk-melcepstrum");
  sig.nframes  = 0;      /* dummy value */
  write_fe_header(out_fp, &sig);

  /* read in samples */
  read_size = read(in_fd, raw_data, 4 * nsamples * samp_size); /* 4 byte reals */

  /* write out coefficients, then the ttl energy */
  for (i = 0 ; i < nsamples; i ++) {
    write_fe_frame_number(out_fp, i);
    for (j = 0 ; j < nfeatures; j++)
      write_fe_bin(out_fp, raw_data[i*(nfeatures+1) + j]);  
    write_fe_energy(out_fp, raw_data[i*(nfeatures+1) + nfeatures ]);
  }
  fflush(out_fp);
}

/*-----------------------------------------------------------------------------*/  
