/* file = judge.c */ 
/* compare a file describing a lattice of word hypotheses (.wh or .gl files) 
   to the veridical labeling (a .la file)
   and output a score */
/* Nigel Ward, University of Tokyo, completely redone in March 1995 */

/*-----------------------------------------------------------------------------*/
#include "jmeat.h"

static int verbose;    

static int nfiles;
static int actual_nfiles;
static int active_file;
static char file_list[100][MAX_PATH_LEN];
static char label_dir[MAX_PATH_LEN];   /* directory in which to seek .la files */

/*-----------------------------------------------------------------------------*/

int process_judge_options(int argc, char *argv[]);
void judge_usage(void);
void main(int argc, char *argv[]);
void print_timeline(float *timeline);

/*-----------------------------------------------------------------------------*/
int process_judge_options(int argc, char *argv[])
{
  int i, last_arg_read;
  last_arg_read = 0;
  strcpy(label_dir, "");

  for (i = 1; i < argc && argv[i][0] == '-'; i++) {
    switch(argv[i][1]) {
    case 'r': case 'l':  i++;  strcpy(label_dir, argv[i]);  break;
    case 'v':  verbose = TRUE;  break; 
    default:  
      fprintf(stderr, "   (%c is not a known option)\n", argv[i][1]);
      judge_usage(); exit(STRANGE); break;
    }
    last_arg_read = i;
  }
  return(last_arg_read);
}

/*-----------------------------------------------------------------------------*/
void judge_usage(void)
{
  fprintf(stderr, "usage: \n");
  fprintf(stderr, "   jlat [-v] [-r reference_directory] [word_hypothesis_files]+\n");
  fprintf(stderr, "Options \n");
  fprintf(stderr, "  -v         verbose\n"); 
  fprintf(stderr, "  -r <dir>   specify directory to use for .la files\n");
}

/*-----------------------------------------------------------------------------*/
void main(int argc, char *argv[])
{
  int i, first_wh_file;

  first_wh_file = 1 + process_judge_options(argc, argv);
  
  nfiles = argc - first_wh_file;
  actual_nfiles = nfiles;
  if (nfiles == 0) {
    fprintf(stderr, "please specify one or more .wh or .gl files\n");
    judge_usage();
    exit(STRANGE);}
  for (i = 0; i < nfiles; i++)
    strcpy(file_list[i], argv[first_wh_file + i]);

  for(active_file = 0; active_file < nfiles; active_file++) {
    if(verbose)
      printf("\nFor %s --------------------------------------\n", file_list[active_file]);
    else
      printf("\n%-30s ", file_list[active_file]);
    input_length = read_la_file_from_dir(label_dir, file_list[active_file], &nlabels, labels, FALSE);
    if (nlabels == 0) {
      printf( "\n  !missing, invalid, or empty label file --- skipping file `%s'\n",
	     file_list[active_file]);
      actual_nfiles --;
      continue;  }
    (void) read_wh_file(file_list[active_file], &nwh, hyps);
    if (nwh == 0) {
      fprintf(stderr, "no word hypotheses in file; skipping it");
      actual_nfiles --;
      continue; }
    if (input_length == 0) {
      fprintf(stderr, "Sorry, input_length is not known, exiting");
      exit(STRANGE); }
    if (verbose) printf("  input_length=%d,  %d word-hypotheses \n", input_length, nwh);

    do_judge();
    if(verbose) {
      printf("correctness = ");
      print_timeline(correctness);
      printf("sum_of_evid = ");
      print_timeline(sum_evidence);
    }
    printf("%s", score_string());
  }
  if (nfiles > 1)
    printf("\n >>>> %7.3f <<<< (grand average over all %d lattice files)",
	   grand_total / actual_nfiles, actual_nfiles);
  printf("\n");
}


/*-----------------------------------------------------------------------------*/
void print_timeline(float *timeline)
{
  int i;
  for(i = 0; i < njudgepoints; i++)
    printf("%.2f ", timeline[i]);
  printf("\n");
}


/*-----------------------------------------------------------------------------*/
