/* file = jmeat.c */ 
/* Nigel Ward, University of Tokyo, April 1994, Jan 1995 */

#include "jmeat.h"

/*-----------------------------------------------------------------------------*/

static char label_cache[MAX_JUDGEPOINTS][MAX_WORD_LEN];
char unlabeled_flag[] = "xUNLABELEDx";

/*=============================================================================*/
void initialize(void);
void setup_label_cache(void);
struct timeline *find_wordline(char *word);
void normalize_wordlines(void); 
void compute_correctness(void);

char *score_string(void);

/*=============================================================================*/
void do_judge(void) 
{
  initialize();
  setup_wordlines();
  normalize_wordlines();
  compute_correctness();
  grand_total += score;
}

/*-----------------------------------------------------------------------------*/
void initialize(void)
{
  int i, j;

 njudgepoints = input_length / JUDGE_INTERVAL;
  nvocab = 0;
  for(i = 0; i < MAX_VOCAB; i++) {
    wordline[i].word[0] = NULL;  /* an extra precaution */
    for(j = 0; j < MAX_JUDGEPOINTS; j++) {
      wordline[i].evidence[j] = 0.0; } }
  for(j = 0; j < MAX_JUDGEPOINTS; j++) {
    sum_evidence[j] = 0.0;
    correctness[j] = 0.0; }
  setup_label_cache(); 
}

/*-----------------------------------------------------------------------------*/
int first_judgepoint_after(int ms)
{
  int tmp;
  tmp = ( (int) (ms + JUDGE_INTERVAL - 1) / JUDGE_INTERVAL); 
  /* printf("rounding %d up to %d,  ", ms, tmp ); */
  return(tmp);
}

/*-----------------------------------------------------------------------------*/
void setup_wordlines(void) 
{
  struct timeline *tl_ptr;  int i, t;

  for (i = 0; i < nwh; i++) {
    tl_ptr = find_wordline(hyps[i].label);
    for (t = first_judgepoint_after(hyps[i].start); 
	 t < first_judgepoint_after(hyps[i].end); 
	 t ++) {
      tl_ptr->evidence[t] += hyps[i].score;
      sum_evidence[t] += hyps[i].score; } }
  /* printf("  %d words seen in hypotheses \n", nvocab); */
}

/*-----------------------------------------------------------------------------*/
/* find or allocate timeline */
struct timeline *find_wordline(char *word)
{
  int i;

  for(i = 0; i < nvocab; i++) {
    if(strcmp(word, wordline[i].word) == 0) {
      /* printf("found %s at %d \n", word, i);   */
      return &(wordline[i]); } }
  if (nvocab >= MAX_VOCAB) {
    fprintf(stderr,"too many different words, exiting\n");
    exit(STRANGE); }
  /* printf("storing %s at %d \n", word, nvocab);  */
  strcpy(wordline[nvocab].word, word);
  nvocab ++;
  return &(wordline[nvocab - 1]);
}
    
/*-----------------------------------------------------------------------------*/
/* adjust things so that the maximum sum of evidence at any time point is 1.0 */
void normalize_wordlines(void) 
{
  float max_evidence;  int i, t;
  
  max_evidence = 0.0;
  for (t = 0; t < njudgepoints; t ++)
    if (sum_evidence[t] > max_evidence)
      max_evidence = sum_evidence[t];
  for (i = 0; i < nvocab; i++)
    for (t = 0; t < njudgepoints; t ++)
      wordline[i].evidence[t] = wordline[i].evidence[t] / max_evidence;
  for (t = 0; t < njudgepoints; t ++)
    sum_evidence[t] = sum_evidence[t] / max_evidence;
}

/*-----------------------------------------------------------------------------*/
void compute_correctness(void)
{
  int jp, labeled_judgepoints;
  char *label;
  struct timeline *relevant_tl_ptr;
  float in_the_money;

  score = 0.0;
  labeled_judgepoints = 0;
  for (jp = 0; jp < njudgepoints; jp++) {
    label = label_at_time(jp);
    /* printf("at time %d word=`%s'; ", t, label); */
    if (strcmp(label, "xUNLABELEDx") != 0) {
      relevant_tl_ptr = find_wordline(label);
      in_the_money = relevant_tl_ptr->evidence[jp];
      if (in_the_money == 0.0) 
	correctness[jp] = 0.0;
      else
	correctness[jp] = in_the_money / sum_evidence[jp];
      /* printf("at %d, in money %.3f, sum %.3f, corr %.2f \n", 
	 jp, in_the_money, sum_evidence[jp], correctness[jp]); */

      score += correctness[jp];
      labeled_judgepoints ++; }
  }
  score = score / labeled_judgepoints;
}

/*-----------------------------------------------------------------------------*/
void setup_label_cache(void)
{
  int i, point, first_jp, terminating_jp;

  for(point = 0; point < njudgepoints; point++)  
    strcpy(label_cache[point], unlabeled_flag);
  
  for (i = 0; i < nlabels; i++) {
    first_jp = first_judgepoint_after(labels[i].start);
    terminating_jp = first_judgepoint_after(labels[i].end);
    /* fprintf(stderr, "for label %s: first_jp %d, terminating_jp %d\n", 
	    labels[i].label, first_jp, terminating_jp); */
    for(point = first_jp; point < terminating_jp; point++) {
      /* fprintf(stderr, " %d ", point); */
      strcpy(label_cache[point], labels[i].label); }
  } 
  /* for(i = 0; i < njudgepoints; i++)  
    fprintf(stderr, "label at %d is %s,  ", i, label_cache[i]); */
}

/*-----------------------------------------------------------------------------*/
char *label_at_time(int judgepoint)
{
  return(label_cache[judgepoint]);
}

/* ----------------------------------------------------------------------------- */
char *score_string(void)
{
  static char result_string[100];
  sprintf(result_string, "lattice_correctness = %.3f", score);
  return(result_string);
}

/*-----------------------------------------------------------------------------*/
/* convert distances to scores (higher is better) */
void invert_scores()
{
  int i;
  float min_dist;

  min_dist = hyps[0].score;
  for (i = 0; i < nwh; i++) {
    if (hyps[i].score < min_dist) 
      min_dist = hyps[i].score;  }
  for (i = 0; i < nwh; i++) {  
    hyps[i].score = min_dist / hyps[i].score;  
  } }

/* ----------------------------------------------------------------------------- */
