/* file = rwin.c */
/* real-time window display of the match scores for each template,
   ie, the wordspotting results  */ 
/* Nigel Ward, University of Tokyo, April 1994 */

#include "cheap.h"
#include "fmeat.h"
#include "xutil.h"
#include "rwin.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* !!!!! this calls routines which use the same global variables (eg win)
   as used by showmatch etc; could be dangerous if someday I want a program
   that uses two display windows at once !!!! */
/*-----------------------------------------------------------------------------*/
int xpos_for_frame(int frame);    

/*-----------------------------------------------------------------------------*/
#define CANVAS_HEIGHT  800
#define YSPACING 20
#define E_YGAIN 4000.0  /* this is approx 50 times a good score */
#define L_YGAIN .8      /* this is approx 50 times a good scoregrep fe_id_st *c
cogsci,inputs:  grep fe_id_st *h
 from log templates */

#define YMARGIN 20 
#define XMARGIN 100 
#define CANVAS_WIDTH (2 * XMARGIN + (MAX_SAMPLES / SAMPLES_PER_PIXEL))
#define BOX_HEIGHT 16

/*-----------------------------------------------------------------------------*/
void prepare_realtime_canvas(int ntemplates, struct signature templates[])
{ int i;

  sg_create(CANVAS_WIDTH, CANVAS_HEIGHT, FALSE, "wordstart likelihoods"); 
  /* want to be sure window is up before drawing, let alone reading audio */
  sg_wait_for_mapping(); 

  /* sg_clear(); */
  /* mark the x axis by drawing vertical lines every one second */
  for (i = 0; i <= MAX_SECONDS; i++) {
    /* 100 frames per second */
    sg_line(xpos_for_frame(100 * i), 0, xpos_for_frame(100 * i), CANVAS_HEIGHT); } 
  draw_y_axis(2, ntemplates);
  draw_template_labels(2, ntemplates, templates);
  sg_flush();
}

void draw_template_labels(int x, int ntemplates, struct signature templates[])
{ int i;
  for (i =0; i < ntemplates; i++) {
    sg_string(6 + x, YMARGIN + (1 + i) * YSPACING + (BOX_HEIGHT - 2) ,
	      templates[i].info_string); } }

void clean_up_realtime_canvas(void)
{ sg_close();}

void draw_right_y_axis(int ntemplates, int input_len)     
{  draw_y_axis(10 + xpos_for_frame(input_len), ntemplates); }

void draw_right_template_labels(int ntemplates, struct signature templates[], 
				int input_len)
{  draw_template_labels(10 + xpos_for_frame(input_len), ntemplates, templates); }

void draw_y_axis(int x1, int ntemplates)     
{
  int i;
  for (i = 0; i < ntemplates; i++)
    sg_outline_rect(x1, YMARGIN + (1 + i) * YSPACING, (XMARGIN - 2) - 2, BOX_HEIGHT); 
}

void draw_likelihood(int frame, int tnum, float yvalue, int logp)
{ int x, y0, y1;
  x = xpos_for_frame(frame);
  y0 = YMARGIN + (1 + tnum) * YSPACING + BOX_HEIGHT;
  if (logp)
    y1 = y0 - (int) (L_YGAIN / yvalue);
  else
    y1 = y0 - (int) (E_YGAIN / yvalue);
  sg_line(x, y0, x, y1);
  /* draw a white shadow on top of the bar,
     so it's still visible even if it's so tall that it runs into the next bar up */
  sg_preset(x, y1 - 1);   sg_preset(x, y1 - 2);   sg_preset(x, y1 - 3);
}

/* 2 pixel spacing between frames */
int xpos_for_frame(int frame)    
{  return(XMARGIN + frame * 2);}


/* ============================================================================= */
