BABYL OPTIONS:
Version: 5
Labels:
Note:   This is the header of an rmail file.
Note:   If you are seeing it in rmail,
Note:    it means the file has no messages in it.

1,,
Received: from alpha.xerox.com by kujira.sanpo.t.u-tokyo.ac.jp (4.1/r2TMx)
	id AA06860; Thu, 16 Mar 95 09:06:26 JST
Received: from IMPACT.Xerox.COM ([13.2.144.1]) by alpha.xerox.com with SMTP id <14531(5)>; Wed, 15 Mar 1995 16:06:09 PST
Received: from lonewolf.IMPACT.Xerox.COM by IMPACT.Xerox.COM (4.1/SMI-4.1)
	id AA11038; Wed, 15 Mar 95 16:06:05 PST
Received: by lonewolf.IMPACT.Xerox.COM (5.x/SMI-SVR4)
	id AA00801; Wed, 15 Mar 1995 16:06:04 -0800
Date: Wed, 15 Mar 1995 16:06:04 PST
From: bj@impact.xerox.com (Bill Jackson)
Message-Id: <9503160006.AA00801@lonewolf.IMPACT.Xerox.COM>
To: nigel@sanpo.t.u-tokyo.ac.jp
Subject: ...spectrum.c
Content-Type: X-sun-attachment

*** EOOH ***
Date: Wed, 15 Mar 1995 16:06:04 PST
From: bj@impact.xerox.com (Bill Jackson)
To: nigel@sanpo.t.u-tokyo.ac.jp
Subject: ...spectrum.c
Content-Type: X-sun-attachment

----------
X-Sun-Data-Type: text
X-Sun-Data-Description: text
X-Sun-Data-Name: text
X-Sun-Charset: us-ascii
X-Sun-Content-Lines: 14


Nigel,
	I just realized that I did some 'touch-up' work to spectrum.c
	which you should pick up if you want to include this "hack"
	in lotec.  I've forgotten what was in the version I last
	sent you, but I think that the new things in this version
	are WARNING & ASSERT macros and 'getopt' command line parsing.

	I've refrained from putting in any copyright since this
	is derived from your code, and because I don't feel a need
	to add any kind of restrictions.  Feel free to add anything
	you'd like in this regard.
-bj

----------
X-Sun-Data-Type: c-file
X-Sun-Data-Name: spectrum.c
X-Sun-Charset: us-ascii
X-Sun-Content-Lines: 556

/*
 * Power Spectrum & Musical Scale Analysis
 */

#ident	"%Z%%M%	%I%	%E%"

#include <sys/types.h>
#include <fcntl.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <multimedia/audio_encode.h>
#include <multimedia/audio_filehdr.h>
#include <multimedia/audio_hdr.h>

extern char *optarg;
extern void FAST(float *fftd, int n);

static int _hash = 0;
static int _debug = 0;
static int _verbose = 0;
static char *_powerfile = "power.pgm";
static char *_guitarfile = "guitar.pgm";

#define	DPRINTF2(fmt, a, b)	if (_debug) fprintf(stderr, fmt, a, b);
#define	VPRINTF1(fmt, a)	if (_verbose) fprintf(stderr, fmt, a);

#define	SAMPLES_PER_FRAME	256


#define	ASSERT(EX)	((void)((EX) || assfail(#EX, __FILE__, __LINE__)))
#define	WARNING(EX)	((void)((EX) && asswarn(#EX, __FILE__, __LINE__)))

int
assfail(char *expr, char *file, int line)
{
	fprintf(stderr, "%s - line %d: ASSERT(%d) failed.\n", file, line, expr);
	exit (EXIT_FAILURE);
	/* NOTREACHED */
	return (1);
}

int
asswarn(char *expr, char *file, int line)
{
	fprintf(stderr, "%s - line %d: WARNING(%d).\n", file, line, expr);
	return (0);
}

typedef struct {
	int d_len;
	void *d_buf;
} desc_t;


static desc_t
malloc_floatarray(int nsamples)
{
	desc_t d;
	float *bp = (float *) malloc(nsamples * sizeof (float));

	if (bp == NULL) {
		perror("malloc");
		exit (EXIT_FAILURE);
	}

	d.d_len = nsamples;
	d.d_buf = bp;
	return (d);
}

static void
dump_hdr(FILE *fd, Audio_filehdr *h)
{
	fprintf(fd, "magic=%x\n", h->magic);
	fprintf(fd, "hdr_size=%d\n", h->hdr_size);
	fprintf(fd, "data_size=%d\n", h->data_size);
	fprintf(fd, "encoding=%d\n", h->encoding);
	fprintf(fd, "sample_rate=%d\n", h->sample_rate);
	fprintf(fd, "channels=%d\n", h->channels);
}

static desc_t
get_audio_data(char *filename, Audio_filehdr *h)
{
	int fd = open(filename, O_RDONLY);
	int info_bytes;
	unsigned char *bp;
	int nsamples;
	int rc;
	desc_t desc;

	if (fd < 0) {
		perror(filename);
		exit (EXIT_FAILURE);
	}

	rc = read(fd, h, sizeof (Audio_filehdr));
	if (rc != sizeof (Audio_filehdr)) {
		perror("read error");
		exit (EXIT_FAILURE);
	}

	if (_debug) dump_hdr(stderr, h);

	if (h->magic != AUDIO_FILE_MAGIC) {
		fprintf(stderr, "%s: bad magic number\n", filename);
		perror("read error");
		exit (EXIT_FAILURE);
	}

	info_bytes = h->hdr_size - sizeof (Audio_filehdr);
	if (info_bytes > 0) {
		char *info_string = malloc(info_bytes+1);

		rc = read(fd, info_string, info_bytes);
		if (rc != info_bytes) {
			perror("read error");
			exit (EXIT_FAILURE);
		}

		DPRINTF2("info=(%d)<%s>\n", info_bytes, info_string);
		free(info_string);
	}

	nsamples = h->data_size;

	bp = (unsigned char *) malloc(nsamples);
	if (bp == NULL) {
		perror("malloc");
		exit (EXIT_FAILURE);
	}

	rc = read(fd, bp, nsamples);
	if (rc != nsamples) {
		perror("read error");
		exit (EXIT_FAILURE);
	}

	close(fd);

	desc.d_buf = bp;
	desc.d_len = nsamples;
	return (desc);
}

static void
EnergyFFT(float *data, int n, desc_t spectrum)
{
	float *result = spectrum.d_buf;
	float *complex_fft = (float *) malloc(n * sizeof (float));
	int i;
	int k;

	ASSERT(spectrum.d_len >= (n / 2));
	for (i = 0; i < n; i++) {
		complex_fft[i] = data[i];
	}

	FAST(complex_fft, n);

	for (i = 0, k = 0; k < n; i++, k += 2) {
		double re = complex_fft[k];
		double im = complex_fft[k + 1];
		double sum = (re * re) + (im * im);

		result[i] = (float) sqrt(sum);
	}

	free(complex_fft);
}

#ifndef	lint
static int
compare(const void *p1, const void *p2)
{
	float t1 = *(float *) p1;
	float t2 = *(float *) p2;

	if (t1 == t2) return (0);
	if (t1 > t2) return (1);
	return (-1);
}

static void
dump_int_data(FILE *fd, int *bp, int n)
{
	int i;

	for (i = 0; i < n; i ++) {
		int sample = bp[i];
		fprintf(fd, " %d", sample);
	}

	fprintf(fd, "\n");
}

static void
dump_float_data(FILE *fd, float *bp, int n)
{
	int i;

	for (i = 0; i < n; i ++) {
		float sample = bp[i];
		fprintf(fd, " %4.2f", sample);
	}

	fprintf(fd, "\n");
}

static void
dump_sorted(FILE *fd, float *bp, int n)
{
	float *sorted = malloc(n * sizeof (float));
	int i;

	for (i = 0; i < n; i++) { sorted[i] = bp[i]; }
	qsort(sorted, n, sizeof (float), compare);
	dump_float_data(fd, sorted, n);
	free(sorted);
}

static void
logscale_data(float *bp, int n, float *scaled, float *max)
{
	int i;
	float biggie = 0.0;

	for (i = 0; i < n; i++) {
		float raw = bp[i];
		float shrunk = (raw > 1.0) ? log(raw) : 0.0;	/* avoid negative values */
		scaled[i] = shrunk;
		if (shrunk > biggie) biggie = shrunk;
	}

	if (max != NULL) *max = biggie;
}
#endif	/* lint */

static desc_t
convert_u2c(desc_t audio_data)
{
	int nsamples = audio_data.d_len;
	unsigned char *bp = audio_data.d_buf;
	float *data = (float *) malloc (nsamples * sizeof (float));
	desc_t d;
	int i;

	for (i = 0; i < nsamples; i++) {
		unsigned char sample = bp[i];
		unsigned short linear = audio_u2s(sample);
		float big = (float) linear;

		data[i] = big / 65536.0;
	}

	d.d_len = nsamples;
	d.d_buf = data;
	return (d);
}

#ifndef	lint
static void
hamm_smooth(float *window, int width, float *smooth)
{
	int i;

	ASSERT(width != 0);
	for (i = 0; i < width; i++) {
		float sample = window[i];
		float fraction = (float) i / (float) width;
		float contrib = cos(2.0 * M_PI * fraction);
		float hamm = 0.46 * contrib;
		smooth[i] = sample * (.54 - hamm);
	}

}

/*
 * a overlapping set of sawtooth filters
 * the center freq of one band is the upper bound of the band to the left,
 * and is also the lower bound of the band to the right
 */
static float sawtooth[] = {150.0, 350.0, 550.0, 800.0, 1050.0, 1350.0, 1700.0, 2300.0, 3000.0, 3900.0};
static float major_scale_C8[] = {3951.0, 4186.0, 4435.0, 4699.0, 4978.0, 5274.0, 5588.0, 5920.0, 6272.0, 6645.0, 7040.0, 7459.0, 7902.0, 8372.0};
	/* b7, c8, c#, d, d#, e, f, f#, g, g#, a, a#, b, c9 */
#endif	/* lint */

int
filter2index(desc_t filter, int chan, float frames_per_sec)
{
	float *fcn = filter.d_buf;
	float hertz = fcn[chan];

	ASSERT(chan <= filter.d_len);
	ASSERT(frames_per_sec != 0);

	return ((int) (hertz / frames_per_sec));
}

/*
 * since we use triangle filters,
 * ttl is slightly greater than the sum of the channels
 */
static float
spectrum2channels(desc_t spectrum, float frames_per_sec, desc_t channels, desc_t filter)
{
	int nsamples = spectrum.d_len;
	int nchan = channels.d_len;
	float *sp = spectrum.d_buf;
	float *cp = channels.d_buf;
	float total = 0.0;
	int i;

	ASSERT(nchan <= filter.d_len);

	for (i = 0; i < nchan; i++) {
		float bin_sum = 0.0;
		int lower = filter2index(filter, i, frames_per_sec);
		int center = filter2index(filter, i+1, frames_per_sec);
		int upper = filter2index(filter, i+2, frames_per_sec);
		int below = center - lower;
		int above = upper - center;
		int j;

		/* if (freq < too_slow) || (freq > too_fast)) { */
		if ((center < 1) || (upper > nsamples)) {
			cp[i] = bin_sum;
			continue;
		}

		for (j = lower; j <= center; j++) {
			float sample = sp[j];
			int width = j - lower;
			float portion = (below == 0) ? 0.0 : (float) width / (float) below;

			bin_sum += portion * sample;
			total += sample;
		}

		for (j = center; j < upper; j++) {
			float sample = sp[j];
			int width = j - center;
			float portion = (above == 0) ? 0.0 : (float) width / (float) above;

			bin_sum += portion * sample;
		}

		cp[i] = bin_sum;
	}

	return (total);
}

static void
color_code(FILE *fd, float *bp, int n, int blips)
{
	int i;

	for (i = 0; i < n; i++) {
		char *fmt = ((i % 12) == 11) ? "%d\n" : "%d ";
		float sample = (bp == NULL) ? 64.0 : bp[i];
		unsigned char byte = (sample > 255.0) ? 0 : 255 - (unsigned char) sample;

		if ((blips) && ((i % 12) == 0)) fprintf(fd, "%d ", 128); /* b# */
		if ((blips) && ((i % 12) == 5)) fprintf(fd, "%d ", 192); /* e# */
		fprintf(fd, fmt, byte);
	}

	fprintf(fd, "\n");
}

static desc_t
build_filter(int base_freq, int octaves)
{
	float delta = 1.059465;	/* donno why this number?  - 2^(1/12) = 1.059463094 */
	int ncell = (octaves * 12) + 2;
	desc_t filter = malloc_floatarray(ncell);
	float *fp = filter.d_buf;
	float y;
	int i;

	for (i = ncell - 1, y = base_freq * delta; i >= 0; i--, y /= delta) {
		fp[i] = y;
	}

	return (filter);
}

static void
compute_power(desc_t linear_data, int sampling_rate, int frame_size)
{
	int nsamples = linear_data.d_len;
	int limit = nsamples - frame_size;
	int nframes = limit / frame_size;
	float fps = (float) sampling_rate / (float) frame_size;
	int tick = fps;
	int secs = nframes / tick;
	float *bp = linear_data.d_buf;
	FILE *pfd = fopen(_powerfile, "w");
	FILE *nfd = fopen(_guitarfile, "w");
	/* desc_t smooth = malloc_floatarray(frame_size); */
	desc_t spectrum = malloc_floatarray(frame_size / 2);
	int octaves = 6;
	desc_t piano_scale = build_filter(4186, octaves);	/* (8372, 9) */
	desc_t notes = malloc_floatarray(octaves * 12);
	float max = 0.0;
	float ttl = 0.0;
	int i;

	ASSERT(frame_size != 0);

	fprintf(pfd, "P2\n");
	fprintf(pfd, "# power.pgm - power spectrum analysis of an audio file\n");
	fprintf(pfd, "# portable graymap format - ala pgm(5)\n");
	fprintf(pfd, "%d %d 255\n", spectrum.d_len - 1, nframes);

	fprintf(nfd, "P2\n");
	fprintf(nfd, "# guitar.pgm - musical scale (guitar gamut) analysis of an audio file\n");
	fprintf(nfd, "# portable graymap format - ala pgm(5)\n");
	fprintf(nfd, "%d %d 255\n", notes.d_len + ((_hash) ? (2 * octaves) : 0), nframes + ((_hash) ? secs : 0));

	for (i = 0; i < nframes; i ++) {
		int finger = i * frame_size;
		float *sp = (float *) spectrum.d_buf;
		float total;
		int j;

		/* hamm_smooth(&bp[finger], frame_size, smooth); */
		EnergyFFT(&bp[finger], frame_size, spectrum);
		color_code(pfd, &sp[1], spectrum.d_len - 1, 0);

		if ((_hash) && ((i % tick) == 0)) {
			fprintf(nfd, "# tick \n");
			color_code(nfd, NULL, notes.d_len, _hash);
		}

		total = spectrum2channels(spectrum, fps, notes, piano_scale);
		color_code(nfd, notes.d_buf, notes.d_len, _hash);

		ttl += total;

		for (j = 1; j < spectrum.d_len; j++) {
			float sample = sp[j];
			if (sample > max) max = sample;
		}
	}

	fclose(pfd);

	VPRINTF1("max=%3.2f\n", max);
	VPRINTF1("ttl=%3.2f\n", ttl);
	VPRINTF1("nframes=%d\n", nframes);
	VPRINTF1("secs=%d\n", secs);

	free(notes.d_buf);
	/* free(smooth.d_buf); */
	free(spectrum.d_buf);
}

static void
doit(char *fname)
{
	Audio_filehdr h;
	desc_t audio_data;
	desc_t linear_data;
	int sampling_rate;
	int frame_size = SAMPLES_PER_FRAME;
	audio_data = get_audio_data(fname, &h);
	linear_data = convert_u2c(audio_data);

	sampling_rate = h.sample_rate;

	WARNING(sampling_rate != 8000);
	WARNING(h.encoding != AUDIO_FILE_ENCODING_MULAW_8);
	WARNING(h.channels != 1);

	compute_power(linear_data, sampling_rate, frame_size);
}

static char *
basename(char *path)
{
	char *p;

	if (path == NULL) {
		return (path);
	}

	for (p = path + strlen(path); (p != path); p--) {
		char ch = *p;

		if (ch == '/') {
			return (p + 1);
		}
	}

	return (path);
}

static void
usage(char *pgm)
{
	fprintf(stderr, "usage: %s [-h] [-d] [-s] [-v] [-g guitar.pgm] [-p power.pgm] <file>\n", pgm);
}

/*ARGSUSED*/
int
main(int argc, char *argv[])
{
	char *pgm = basename(argv[0]);
	int c;
	char *fname = NULL;

	while ((c = getopt(argc, argv, "dvg:p:")) != EOF) {
		switch (c) {
		case 'h':
			_hash++;
			break;
		case 'd':
			_debug++;
			break;
		case 's':
			_verbose = 0;
			break;
		case 'v':
			_verbose++;
			break;
		case 'g':
			_guitarfile = optarg;
			break;
		case 'p':
			_powerfile = optarg;
			break;
		default:
			usage(pgm);
			exit (EXIT_FAILURE);
		}
	}

	if (optind < argc) {
		fname = argv[optind];
	}

	if (fname == NULL) {
		usage(pgm);
		exit (EXIT_FAILURE);
	}

	doit(fname);

	return (EXIT_SUCCESS);
}

/* eof */

