/* file = xutil.c */
/* simple code for using creating and using an window under X */
/* Nigel Ward, University of Tokyo, April 1994; based on code by Minoru Terada */
/*------------------------------------------------------------------*/

#include "cheap.h"
#include "xutil.h"

static int	screen_num;
static Screen	*screen_ptr;
static int	x, y;
static int	border_width=4;
static GC	gc_fore, gc_back, gc_red;
static int	doflush;
static XColor   cdef;

void sg_create(int width, int height, int flushflag, char *name)	
{
  Colormap default_cmap;

	display = XOpenDisplay(NULL);        
	screen_num = DefaultScreen(display);
	screen_ptr = DefaultScreenOfDisplay(display);

	win = XCreateSimpleWindow(
		display,
		RootWindow(display, screen_num),
		x, y,
		width, height,
		border_width,
		BlackPixel(display, screen_num),
		WhitePixel(display, screen_num));

	XStoreName(display,win,name);

	XMapWindow(display, win);

	gc_fore = XCreateGC(display, RootWindow(display, screen_num), 0, NULL);
	XSetForeground(display, gc_fore, BlackPixel(display, screen_num));
	XSetBackground(display, gc_fore, WhitePixel(display, screen_num));

	gc_back = XCreateGC(display, RootWindow(display, screen_num), 0, NULL);
	XSetForeground(display, gc_back, WhitePixel(display, screen_num));
	XSetBackground(display, gc_back, BlackPixel(display, screen_num));

        default_cmap = DefaultColormap(display, screen_num);
	XAllocNamedColor(display, default_cmap, "red3", &cdef, &cdef);
	gc_red = XCreateGC(display, RootWindow(display, screen_num), 0, NULL);
        XSetForeground(display, gc_red, cdef.pixel);
        XSetBackground(display, gc_red, WhitePixel(display, screen_num));

	XFlush(display);

	doflush = flushflag;
}

void sg_wait_for_mapping(void)
{
  XSelectInput(display, win, ExposureMask);
  XNextEvent(display, &report);
  fprintf(stderr, " xutil: mapping occured (report.type = %d)\n", report.type);
}

void sg_clear(void)
{	XClearWindow(display,win);
	if(doflush)XFlush(display);}

void sg_line(int x0,int y0,int x1,int y1)
{	XDrawLine(display, win, gc_fore, x0, y0, x1, y1);
	if(doflush)XFlush(display);}

void sg_line_red(int x0,int y0,int x1, int y1)
{	XDrawLine(display, win, gc_red, x0, y0, x1, y1);
	if(doflush)XFlush(display);}

void sg_clear_line(int x0, int y0, int x1, int y1)
{	XDrawLine(display, win, gc_back, x0, y0, x1, y1);
	if(doflush)XFlush(display);}

void sg_string(int x, int y, char *s)	
{	XDrawString(display, win, gc_fore, x, y, s, strlen(s));
	if(doflush)XFlush(display); }

void sg_string_red(int x, int y, char *s)	
{	XDrawString(display, win, gc_red, x, y, s, strlen(s));
	if(doflush)XFlush(display); }
void sg_clear_string(int x, int y, char *s)
{	XDrawString(display, win, gc_back, x, y, s, strlen(s));
	if(doflush)XFlush(display); }

void sg_pset(int x, int y)
{	XDrawPoint(display, win, gc_fore, x, y);
	if(doflush)XFlush(display);}

void sg_pset_red(int x, int y)
{	XDrawPoint(display, win, gc_red, x, y);
	if(doflush)XFlush(display);}

void sg_preset(int x, int y)
{	XDrawPoint(display, win, gc_back, x, y);
	if(doflush)XFlush(display);}

void sg_draw_rect(int x, int y, int width, int height)
{  XFillRectangle(display, win, gc_fore, x, y, width, height);
  if(doflush)XFlush(display);}

void sg_erase_rect(int x, int y, int width, int height)
{  XFillRectangle(display, win, gc_back, x, y, width, height);
  if(doflush)XFlush(display);}

void sg_outline_rect(int x, int y, int width, int height)
{  XDrawRectangle(display, win, gc_fore, x, y, width, height);
  if(doflush)XFlush(display);}

void sg_clear_outline_rect(int x, int y, int width, int height)
{  XFillRectangle(display, win, gc_back, x, y, width, height);
  if(doflush)XFlush(display);}

void sg_flush(void)
{	XFlush(display);}

void sg_close(void)
{	XFreeGC(display,gc_fore);
	XFreeGC(display,gc_back);
	XCloseDisplay(display);}

void sg_bell(void)
{    XBell(display, 100);}

void vertical_dots(int x, int y1, int y2) 
{
  int yy;
  for (yy = y1; yy < y2; yy += 8)
    sg_pset(x,yy); }

void horizontal_dots(int x1, int x2, int y) 
{
  int xx;
  for (xx = x1; xx < x2; xx += 12)
    sg_pset(xx,y); }
